/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.model.constraint.OldInterfaceReference;
import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.Connection;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.ui.properties.ComponentTypePropertySource;

import org.eclipse.ui.views.properties.IPropertySource;

public class ComponentTypeController extends ComposableTypeController {

	public ComponentTypeController(ModelController modelController, ComponentType type) {
		super(modelController, type);
	
	}

	

	public Object[] getChildren() {
		if (!(type instanceof ComponentType))
			return null;
		
		
		Iterator<PartDefinition> parts = type.getParts();
		Iterator<AttributeDefinition> attributes = type.getAttributes();    
		Iterator<InterfaceDefinition> interfaces = ((ComponentType)type).getInterfaces();

		List<BasicController> children = new LinkedList<BasicController>();
		
		while (parts.hasNext()) {
			PartDefinitionController pdc = (PartDefinitionController)modelController.getControllerFactory().createController(parts.next());
			pdc.setParent(this);
			children.add(pdc);
		}
		
		
		while (attributes.hasNext()) {
			AttributeDefinitionController adc = (AttributeDefinitionController) modelController.getControllerFactory().createController(attributes.next());
			adc.setParent(this);
			children.add(adc);
		}
		

		while (interfaces.hasNext()) {
			InterfaceDefinitionController idc = (InterfaceDefinitionController) modelController.getControllerFactory().createController(interfaces.next());
			idc.setParent(this);
			children.add(idc);
		}
		
	
		
		if (type.hasConstraints()) {
			children.add(modelController.getControllerFactory().createConstraintGroupController(type));
		
		}
		
		if (((ComponentType)type).hasConnections()) {
			children.add(modelController.getControllerFactory().createConnectionGroupController((ComponentType)type));
		}
	
		return children.toArray();
	}

	public boolean hasChildren() {
		if (type == null)
			return false;
		
		if (((ComponentType)type).hasInterfaces())
			return true;
		
		if (type.getPartsArray() != null && type.getPartsArray().length > 0)
			return true;
		
		if (type.getAttributes() != null &&  type.getAttributes().hasNext())	
			return true;
		
		return false;
	}
	
	
	
	public int getType() {
		return COMPONENT_TYPE;
	}

	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new ComponentTypePropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public void addInterfaceDefinition(Name definitionName, String[] types, boolean isGrounded, boolean isOptional, boolean isProvides) {
		List typeList = new ArrayList(types.length);
		for (int i = 0; i < types.length; i++)
			typeList.add(modelController.getInterfaceType(types[i]));
		try {
			((ComponentType) type).addInterface(new InterfaceDefinition(definitionName, typeList, isGrounded, isOptional, isProvides));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		modified();
	}

	public void removeInterfaceDefinition(String definitionName, boolean isUndo) {
	
		((ComponentType)type).removeInterfaceDefinition(definitionName);
	}


	public void delete() {
		modelController.removeComponentType(type.getName(), false);
		modified();
		
	}

	public String getImageString() {
		if (isAbstract())
			return BasicController.ABSTRACT_COMPONENT_TYPE_IMAGE_STRING;
		return BasicController.COMPONENT_TYPE_IMAGE_STRING;
	}



	public String getBackgroundString() {
		
		return COMPONENT_TYPE_COLOR_STRING;
	}

	

	public boolean hasInterfaces() {
		return ((ComponentType)type).getInterfaces().hasNext();
	}

	public String[] getInterfaces() {
		Iterator<InterfaceDefinition> it = ((ComponentType)type).getInterfaces();
		List<String> names = new LinkedList<String>();
		
		while (it.hasNext()) {
			names.add(it.next().getName());
		}
		
		return names.toArray(new String[names.size()]);
	}



	public void addConnection(InterfaceDefinition firstInterface, PartDefinition firstPart, InterfaceDefinition secondInterface, PartDefinition secondPart) throws PropertyAlreadyDefinedException {
		
		((ComponentType)type).addConnection(new Connection(new OldInterfaceReference(firstPart, (ComponentType) type, firstInterface), new OldInterfaceReference(secondPart, (ComponentType) type, secondInterface)));
	
		modified();
	}



	public InterfaceDefinition getInterface(String name) {
		return ((ComponentType)type).getInterface(name);
	}



	public void removeConnection(ConnectionController connectionController) {
		((ComponentType)type).removeConnection(connectionController.getConnection());
		modified();
	}



	public void addInterfaceDefinition(InterfaceDefinition d) throws PropertyAlreadyDefinedException {
		((ComponentType)type).addInterface(d);
		
	}
	
	


	
}
