/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.language.parser.ModelParser;
import kumbang.core.model.constraint.AttributeReference;
import kumbang.core.model.constraint.HasPartOfTypeExpression;
import kumbang.core.model.constraint.ReferenceItem;
import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.core.AttributedModel;
import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.core.model.core.KumbangModel;
import kumbang.core.model.core.Model;
import kumbang.core.model.exception.ModelException;
import kumbang.core.model.exception.OnesOwnSupertypeException;
import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.exception.RootAlreadyDefinedException;
import kumbang.core.model.exception.TypeAlreadyDefinedException;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.ElementaryType;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.modelchecker.ModelChecker;
import kumbang.modeller.ui.editor.operations.umlexport.UmlExportUtils;
import kumbang.modeller.ui.event.ModelChangeEvent;
import kumbang.modeller.ui.event.ModelChangeListener;
import kumbang.modeller.ui.event.ModelChangeProvider;
import kumbang.modeller.ui.event.ModelTypeChangedEvent;
import kumbang.modeller.ui.event.ModellerEvent;
import kumbang.modeller.util.KumbangModelPrinter;
import kumbang.util.KumbangLogger;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.ListenerList;

@SuppressWarnings("unchecked")
public class ModelController implements ModelChangeProvider {

	
    private KumbangModel model;
	private ModelParser modelParser;
	private int modifications = 0;
	private File file;
	private ControllerFactory controllerFactory;
	

	private ListenerList listeners = new ListenerList();
	private String modelType;
	
 
    public ModelController() {
        modelParser = new ModelParser();
        KumbangLogger.initLogger();
    }


	public void createModel(String name, String type, IPath path) throws ModelException {	
			model = new KumbangModel(name);
			modelType = type;
			file = path.toFile();
	}


    
	public void loadModel(IPath path) throws ModelException  {
		
		file = path.toFile();
		if (file.length() > 0) {
			Model m;
			try {
				m = modelParser.parseModel(file);
				
			} catch (ModelException e) {
				try {
					ObjectInputStream ois = new ObjectInputStream(new FileInputStream(file));
					Object o = ois.readObject();
					ois.close();
					if (o != null && o instanceof Model)
						m = (Model)o;
				else throw e;
				}	catch (Exception e1) {
					throw e;
				}
			}
				
			if (m instanceof KumbangModel ) {
				model = (KumbangModel)m; 
				modelType = KumbangModel.MODEL_KIND;
			} else {
				model = new KumbangModel(m.getName());	
				if (m instanceof AttributedModel)
					model.setAttributedModel((AttributedModel)m);
				
				if (m instanceof ForfamelModel) {
					model.setFeaturedModel((ForfamelModel) m);
			
					modelType = ForfamelModel.MODEL_KIND;
				}
				else if (m instanceof KoalishModel) {
					model.setComponentedModel((KoalishModel) m);
					modelType = KoalishModel.MODEL_KIND;
				}
			}	
					
			
		} else {			
			model = new KumbangModel(path.removeFileExtension().toFile().getName());
			modelType = KumbangModel.MODEL_KIND;
		}			
	}
	
	public KumbangModel getModel() {
		return model;
	}


	public void createNewComposableType(Name name, boolean isAbstract, Collection<ComposableType> supertypes, boolean isFeature, boolean isRoot) throws TypeAlreadyDefinedException, OnesOwnSupertypeException {
		
		ComposableType type;
		
		if (isFeature)
			type = new FeatureType(name);
		else type = new ComponentType(name);
		
		if (isAbstract)
			type.setAbstract();
		for (Iterator<ComposableType> it = supertypes.iterator(); it.hasNext(); ){
			type.addSupertype(it.next(), true);
		}
		
		if (isFeature) {
			model.addFeatureType((FeatureType)type);
			if (isRoot)
				try {
					setFeatureRoot(name.getName());
				} catch (RootAlreadyDefinedException e) {
					//should never happen
				}
		} 
		else{
			model.addComponentType((ComponentType)type); 
			if (isRoot)
				try {
					setComponentRoot(name.getName());
				} catch (RootAlreadyDefinedException e) {
					//should never happen
				}
		}
		
		modified(new ModelChangeEvent(ModelChangeEvent.TYPE));
		
		
	}


	public void createNewAttributValueType(Name name, HashMap<String, Name> values) throws PropertyAlreadyDefinedException, TypeAlreadyDefinedException {

		AttributeValueType type = new AttributeValueType(name);
		Iterator<Name> it = values.values().iterator();
		while (it.hasNext())
			type.addValue(it.next());
			
		model.addAttributeValueType(type);
		modified(new ModelChangeEvent(ModelChangeEvent.TYPE));
		
	}


	public void createNewInterface(Name name, String[] items) throws TypeAlreadyDefinedException, PropertyAlreadyDefinedException {

		InterfaceType type = new InterfaceType(name);

		
		for (int i = 0; i < items.length; i++)
				type.addFunction(items[i]);
		
		model.addInterfaceType(type);
		modified(new ModelChangeEvent(ModelChangeEvent.TYPE));
	}


	public String[] getInterfaces() {
		Iterator iterator = model.getInterfaceTypes();
		String [] interfaces = new String[model.getInterfaceCount()];
		
		for (int i = 0; i < interfaces.length; i++) {
			interfaces[i] = ((InterfaceType)iterator.next()).getDisplayName();
		}
		return interfaces;
	}


	public String[] getComponentTypes() {
		Iterator iterator = model.getComponentTypes();
		String [] components = new String[model.getComponentCount()];
		
		for (int i = 0; i < components.length; i++) {
			components[i] = ((ComponentType)iterator.next()).getDisplayName();
		}
		return components;
	}


	public void setComponentRoot(String string) throws RootAlreadyDefinedException {
		model.setComponentRoot(model.getComponentType(string));
		modified(new ModelChangeEvent(ModelChangeEvent.COMPONENT));
		
	}

	public String[] getAttributeTypes() {
		Iterator iterator = model.getAttributeValueTypes();
		String [] attributes = new String[model.getAttributeValueCount()];
		
		for (int i = 0; i < attributes.length; i++) {
			attributes[i] = ((AttributeValueType)iterator.next()).getDisplayName();
		}
		return attributes;
	}
	

	public String[] getFeatureTypes() {
		Iterator iterator = model.getFeatureTypes();
		String [] features = new String[model.getFeatureCount()];
		
		for (int i = 0; i < features.length; i++) {
			features[i] = ((FeatureType)iterator.next()).getDisplayName();
		}
		return features;
	}


	public void setFeatureRoot(String string) throws RootAlreadyDefinedException {
		model.setFeatureRoot(model.getFeatureType(string));
		modified(new ModelChangeEvent(ModelChangeEvent.FEATURE));
		
	}


	public boolean isComponentRootSet() {
		return model.getComponentRoot() != null;
	}


	public boolean isFeatureRootSet() {
		return model.getFeatureRoot() != null;
	}
	

	public void removeComponentType(String name, boolean isUndo) {
		
		if (getComponentRoot().getName().equals(name))
			removeComponentRoot(isUndo);
		model.removeComponentType(name);
		modified(new ModelChangeEvent(isUndo));
		
	}


	public boolean isModified() {
		return modifications > 0;
	}


	public void removeFeatureType(String name, boolean isUndo) {
		if (getFeatureRoot().getName().equals(name))
			removeFeatureRoot(isUndo);
		else model.removeFeatureType(name);
		modified(new ModelChangeEvent(isUndo));
	}


	public void removeAttributeValue(String name, boolean isUndo) {
		model.removeAttributeValueType(name);
		modified(new ModelChangeEvent(isUndo));
		
	}


	public void removeInterfaceType(String name, boolean isUndo) {
		model.removeInterfaceType(name);
		modified(new ModelChangeEvent(isUndo));
		
	}


	public void removeComponentRoot(boolean isUndo) {
		model.removeComponentRoot();
		modified(new ModelChangeEvent(ModelChangeEvent.COMPONENT, isUndo));
	}


	public void removeFeatureRoot(boolean isUndo) {
		model.removeFeatureRoot();
		modified(new ModelChangeEvent(ModelChangeEvent.FEATURE, isUndo));
		
	}





	public void addInterfaceDefinition(String componentName, String definitionName, String[] types, boolean isGrounded, boolean isOptional, boolean isProvides) throws PropertyAlreadyDefinedException {
		Collection<InterfaceType> c = new ArrayList<InterfaceType>(types.length);
		for (int i = 0; i < types.length; i++) {
			c.add(model.getInterfaceType(types[i]));
		}
		InterfaceDefinition def = new InterfaceDefinition(new Name(definitionName), c, isGrounded, isOptional, isProvides );
		model.getComponentType(componentName).addInterface(def);
	
		modified(new ModelChangeEvent());
		
	}


	public void removeInterfaceDefinition(String componentName, String definitionName, boolean isUndo) {
		model.getComponentType(componentName).removeInterfaceDefinition(definitionName);
		modified(new ModelChangeEvent());
	}


	public void saveAs(String string, boolean asObject) {
		if (string == null)
			return;
		this.file = new File(string);
		if (asObject)
			saveAsObject();
		else save();
		
	}


	public void save() {

		ensureFileExists();
		
		try {
		
				String modelString = KumbangModelPrinter.getInstance().getAsString(model, modelType);
		
				// Check if model can be parser
			
				PrintWriter writer = new PrintWriter(new FileWriter(file));
				writer.print(modelString);
				writer.flush();
				writer.close();
			
			
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
		modifications = 0;
	}

	public void exportAsUml(String fileName) throws ModelException {
		UmlExportUtils.export(model, controllerFactory, fileName);
	}

	private void ensureFileExists() {
		if (!file.exists())
			try {
				file.createNewFile();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		
	}

	/**
	 * Checks the validity of the model
	 * @return true if the model is valid
	 * false if the model is invalid
	 */
	public boolean checkModel() throws ModelException, IOException {
		ModelChecker mc = new ModelChecker();
		return mc.checkModel(this.model);
	}

	public String getName() {
		if (model != null)
			return model.getName();
		return null;
	}


	public void modified(ModellerEvent event) {
//		Model newModel = null;
//		try {
//			newModel = modelParser.parseModel(KumbangModelPrinter.getInstance().getAsString(model));
//		} catch (ModelException e) {
//		
//			e.printStackTrace();
//			//return;
//		}
//		if (newModel !=null && newModel instanceof KumbangModel)	
//		 model = (KumbangModel) newModel;
//
//		
		
		if (event.isUndo())
			modifications--;
		else modifications++;
		
		
		notify(event);
	}
	


	public ControllerFactory getControllerFactory() {
		if (controllerFactory == null)
			controllerFactory = new ControllerFactory(this);
		return controllerFactory;
	}


	public ComposableType getComposableType(String name) {
		ElementaryType type = model.getType(name, FeatureType.TYPE_KIND);
		if (type != null)
			return (ComposableType) type;
		return (ComposableType) model.getType(name, ComponentType.TYPE_KIND);
	}


	public AttributeValueType getAttributeType(String typeName) {
		return model.getAttributeValueType(typeName);
	}


	public String[] getExcludedFeatureTypes(String typeName) {
		FeatureType type = model.getFeatureType(typeName);
		

		// For the root feature, only the root itself cannot be added
		if (type == model.getFeatureRoot())
			return new String[] {typeName};
		
		List list = type.getAllWholeTypes();
		String[] types = new String[list.size() + 1];
		types[0] = typeName;
		for (int i = 1; i < types.length; i++)
				types[i] = ((ComposableType)list.get(i - 1)).getDisplayName();
		
			
		return types;
	
	}


	public String[] getExcludedComponentTypes(String typeName) {		
		ComponentType type = model.getComponentType(typeName);
		
		// For the root component, only the component itself cannot be added
		if (type == model.getComponentRoot())
			return new String[] {typeName};
		
		List list = type.getAllWholeTypes();
		String[] types = new String[list.size() + 1];
		types[0] = typeName;
		for (int i = 1; i < types.length; i++)
				types[i] = ((ComposableType)list.get(i - 1)).getDisplayName();
		
			
		return types;
	
	}

	public ComponentTypeController[] getComponentTypeControllers() {
		if (model == null)
			return new ComponentTypeController[0];
		
		ComponentTypeController[] types = new ComponentTypeController[model.getComponentCount()];
		Iterator it = model.getComponentTypes();
		for (int i = 0; i < types.length; i++) {
			types[i] = (ComponentTypeController) getControllerFactory().createController(it.next());
		}
		return types;
	}
	
	public Object[] getFeatureTypeControllers() {
		
		if (model == null)
			return new Object[0];
		
		Object[] types = new Object[model.getFeatureCount()];
		Iterator it = model.getFeatureTypes();
		for (int i = 0; i < types.length; i++) {
			types[i] = getControllerFactory().createController(it.next());
		}
		return types;
	}


	public Object[] getAttributeTypeControllers() {
		
		if (model == null)
			return new Object[0];
		
		Object[] types = new Object[model.getAttributeValueCount()];
		Iterator it = model.getAttributeValueTypes();
		for (int i = 0; i < types.length; i++) {
			types[i] = getControllerFactory().createController(it.next());
		}
		return types;
	}


	public Object[] getInterfaceTypeControllers() {
		
		if (model == null)
			return new Object[0];
		
		Object[] types = new Object[model.getInterfaceCount()];
		Iterator it = model.getInterfaceTypes();
		for (int i = 0; i < types.length; i++) {
			types[i] = getControllerFactory().createController(it.next());
		}
		return types;
	}


	public InterfaceType getInterfaceType(String type) {
		return model.getInterfaceType(type);
	}


	public FeatureType getFeatureType(String text) {
		return model.getFeatureType(text);
	}


	public void setModelName(String name) {
		model.setName(name);
		modified(new ModelChangeEvent());
	}



	public void addModelChangeListener(ModelChangeListener listener) {
		listeners.add(listener);
		
	}

	public void removeModelChangeListener(ModelChangeListener listener) {
		listeners.remove(listener); 
		
	}

	public void notify(ModellerEvent event) {
		Object[] all = listeners.getListeners();
		
		for (int i = 0; i < all.length; i++) {
			if (all[i] instanceof ModelChangeListener)
				((ModelChangeListener)all[i]).modelChanged(event);
		}
		
	}


	public int getFeatureTypeCount() {
		return model.getFeatureCount();
	}


	public void setModelType(String type) {
		if (type.equals(modelType))
			return;
		modelType = type;
		modified(new ModelTypeChangedEvent(type));
	}
	
	
	public String getModelType() {
		return modelType;
	}


	public boolean hasFeatureModelData() {
		return model.getFeatureCount() > 0;
	}	
	
	public boolean hasComponentModelData() {
		return model.getComponentCount() > 0 || model.getInterfaceCount() > 0;
	}


	public String getFileName() {
		return file.getName();
	}

	public void saveAsObject() {
		ensureFileExists();
		
		try {
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file));
			oos.writeObject(model);
			oos.close();
			modifications = 0;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	public boolean canBeSavedAsText() {
		
		if (modelType.equals(ForfamelModel.MODEL_KIND))
			return isFeatureRootSet();
		
		if (modelType.equals(KoalishModel.MODEL_KIND))
			return isComponentRootSet();	
		
		return isFeatureRootSet() && isComponentRootSet();
	}


	public ComponentType getComponentRoot() {
		return model.getComponentRoot();
	}

	public ComposableType getFeatureRoot() {
		return model.getFeatureRoot();
	}

	private void addParts(List<StructureReference> refs, List<ReferenceItem> path, ComposableType type, ComposableType parentType, ComponentType root) {
		Iterator<PartDefinition> currentParts = parentType.getParts();
		
		while(currentParts.hasNext()) {
			PartDefinition current = currentParts.next();
			Iterator<ComposableType> types = current.getPossibleTypes();
			ArrayList<ReferenceItem> newPath = new ArrayList<ReferenceItem>(path);
			newPath.add(new ReferenceItem(current.getName()));
			refs.add(new StructureReference(type,  newPath, null, root));
			
			while(types.hasNext()) {
				
				ComposableType currentType = types.next();
				
								
				// Need to include type names only if part has several possible types
				if (current.getPossibleTypesCount() > 1) {			
					ArrayList<ReferenceItem> typeChosen = new ArrayList<ReferenceItem>(path);								
					typeChosen.add(new ReferenceItem(current.getName(), currentType));
					refs.add(new StructureReference(type, typeChosen, null, root));
					addParts(refs, typeChosen, type, currentType, root);
					
				} 
				addParts(refs, newPath, type, currentType, root);
			}
		}
	}


	public List<StructureReference> getAllComponentParts() {
		List<StructureReference> refs = new LinkedList<StructureReference>();
		List<ReferenceItem> path = new ArrayList<ReferenceItem>();
		addParts(refs, path, getComponentRoot(), getComponentRoot(), getComponentRoot());
		return refs;
	}
	
	public List<StructureReference> getAllParts(ComposableType type) {
		
		List<StructureReference> refs = new ArrayList<StructureReference>();
		List<ReferenceItem> path = new ArrayList<ReferenceItem>();

		addParts(refs, path, type, type, null);
			
		return refs;
	}
	
	public List<StructureReference> getAllInterfaces(ComposableType cType) {
		
		ComponentType type;
		if (cType instanceof FeatureType)
			type = getComponentRoot();
		else type = (ComponentType)cType;
		
		List<StructureReference> refs = new ArrayList<StructureReference>();
		ArrayList<ReferenceItem> path = new ArrayList<ReferenceItem>();
		for (Iterator<InterfaceDefinition> it = type.getInterfaces(); it.hasNext();) {
			
			refs.add(new StructureReference(type, path, new ReferenceItem(it.next().getName()), type == getComponentRoot() ? type : null));
		}
		
		
		addInterfaces(refs, path, type, type, type == getComponentRoot() ? type : null);			
		return refs;
	}


	private void addInterfaces(List<StructureReference> refs, List<ReferenceItem> path, ComponentType type, ComponentType parentType, ComponentType root) {
	
		Iterator<PartDefinition> currentParts = parentType.getParts();
		
		while(currentParts.hasNext()) {
		
			PartDefinition current = currentParts.next();
			
			ArrayList<ReferenceItem> newPath = new ArrayList<ReferenceItem>(path);
			newPath.add(new ReferenceItem(current.getName()));
				
			
			
			for(Iterator<ComponentType> types = current.getPossibleTypes(); types.hasNext();) {
				ComponentType currentType = types.next();
				
				for (Iterator<InterfaceDefinition> interfaces = currentType.getInterfaces(); interfaces.hasNext();) {
				
					InterfaceDefinition ifd = interfaces.next();
					StructureReference ref = new StructureReference(type, newPath, new ReferenceItem(ifd.getName()), root) ;
					if (!refs.contains(ref))
						refs.add(ref);
					
				}				
				addInterfaces(refs, newPath, type, currentType, root);
			}
			
		}
		
	}
	
	
	public List<AttributeReference> getAllAttributes(ComposableType type, boolean isImplementation) {
		
		List<AttributeReference> refs = new ArrayList<AttributeReference>();
		ArrayList<ReferenceItem> path = new ArrayList<ReferenceItem>();

		
		
		for (Iterator<AttributeDefinition> it = type.getAttributes(); it.hasNext();)
			refs.add(new AttributeReference(type, it.next().getName()));
		
		
		
		addAttributes(refs, path, type, type, null);
		
		if (isImplementation) {
			for (Iterator<AttributeDefinition> it = getComponentRoot().getAttributes(); it.hasNext();) {
			
				refs.add(new AttributeReference(new StructureReference(type, path, null, getComponentRoot()), it.next().getName(), getComponentRoot()));
			
			}
			addAttributes(refs, path, getComponentRoot(), getComponentRoot(), getComponentRoot());
		}
		return refs;
	}


	private void addAttributes(List<AttributeReference> refs, List<ReferenceItem> path, ComposableType type, ComposableType parentType, ComponentType root) {
	
		Iterator<PartDefinition> currentParts = parentType.getParts();
		
		while(currentParts.hasNext()) {
		
			PartDefinition current = currentParts.next();
			
			ArrayList<ReferenceItem> newPath = new ArrayList<ReferenceItem>(path);
			newPath.add(new ReferenceItem(current.getName()));
				
			
			
			for(Iterator<ComposableType> types = current.getPossibleTypes(); types.hasNext();) {
				ComposableType currentType = types.next();

				if (current.getPossibleTypesCount() > 1) {
							
					ArrayList<ReferenceItem> typeChosen = new ArrayList<ReferenceItem>(path);								
					typeChosen.add(new ReferenceItem(current.getName(), currentType));
					for (Iterator<AttributeDefinition> attributes = currentType.getAttributes(); attributes.hasNext();) {
						
						AttributeDefinition ifd = attributes.next();
						refs.add(new AttributeReference(new StructureReference(type,  typeChosen, null, root), ifd.getName()));
						
					}
					addAttributes(refs, typeChosen, type, currentType, root);
					
					
				} else {
				
					for (Iterator<AttributeDefinition> attributes = currentType.getAttributes(); attributes.hasNext();) {		
						AttributeDefinition ifd = attributes.next();
						refs.add(new AttributeReference(new StructureReference(type,  newPath, null, root), ifd.getName()));
					
					}				
					addAttributes(refs, newPath, type, currentType, root);
				}
			}
			
		}
		
	}


	public PartDefinition getPartDefinition(StructureReference ref) {
		
		if (ref == null)
			return null;
		
		PartDefinition currentPart = null;
		for (int i = 0; i < ref.getDepth(); i++) {
			String partName = ref.getName(i);
			if (currentPart == null)
				currentPart = ref.getOrigin().getPart(partName);
			else {
				for (Iterator<ComposableType> it = currentPart.getPossibleTypes(); it.hasNext();) {
					ComposableType c = it.next();
					if (c.getPart(partName) != null)
						currentPart = c.getPart(partName);
					
				}
			}
			
		}
		return currentPart;
	}
	
	public InterfaceDefinition getInterfaceDefinition(StructureReference ref) {
		
		if (!ref.isInterfaceReference())
			return null;
		
	
		PartDefinition currentPart = null;
		if (ref == null)
			return null;
		
		for (int i = 0; i < ref.getDepth(); i++) {
			String partName = ref.getName(i);
			if (currentPart == null)
				currentPart = ref.getOrigin().getPart(partName);
			else {
				for (Iterator<ComposableType> it = currentPart.getPossibleTypes(); it.hasNext();) {
					ComposableType c = it.next();
					if (c.getPart(partName) != null)
						currentPart = c.getPart(partName);
					
				}
			}
			
		}
		
		if (currentPart == null && ref.getOrigin() instanceof ComponentType) {
			for (Iterator<InterfaceDefinition> i = ((ComponentType)ref.getOrigin()).getInterfaces(); i.hasNext();) {
				InterfaceDefinition def = i.next();
				if (def.getName().equals(ref.getInterfaceName()))
					return def;
			}
		}
		
		for (Iterator<ComponentType> it = currentPart.getPossibleTypes(); it.hasNext();) {
			for (Iterator<InterfaceDefinition> i = it.next().getInterfaces(); i.hasNext();) {
				InterfaceDefinition def = i.next();
				if (def.getName().equals(ref.getInterfaceName()))
					return def;
			}
		}
		return null;
	}


	public List<StructureReference> getPossiblePresentTypes(ComposableType type, boolean isImplementation) {
		
		List<StructureReference> refs = new ArrayList<StructureReference>();
		ArrayList<ReferenceItem> path = new ArrayList<ReferenceItem>();
		ComponentType componentType = null;
		
		// Types
		addParts(refs, path, type, type, null);
		
		// Component types from the component root
		if (isImplementation) {
//			path.add(new ReferenceItem("component-root"));
			addParts(refs, path, getComponentRoot(), getComponentRoot(), getComponentRoot());
		
		}
		if (type instanceof ComponentType)
			componentType = (ComponentType) type;
		else if (isImplementation)
			componentType = getComponentRoot();
	
		//Interfaces
		if (componentType != null) {
			
			for (Iterator<InterfaceDefinition> it = componentType.getInterfaces(); it.hasNext();) {		
				refs.add(new StructureReference(componentType, path, new ReferenceItem(it.next().getName()), componentType == getComponentRoot() ? componentType : null));
			}	
			addInterfaces(refs, path, componentType, componentType, componentType == getComponentRoot() ? componentType : null);			
		}
		return refs;
	}


	public List<HasPartOfTypeExpression> getPotentialPartTypes(ComposableType type, boolean isImplementation) {
		ArrayList<HasPartOfTypeExpression> list = new ArrayList<HasPartOfTypeExpression>();
	
		for (Iterator<ComposableType> it = type.getPartTypes().iterator(); it.hasNext();) {
			list.add(new HasPartOfTypeExpression(it.next(), false));
		}
		
		for (Iterator<PartDefinition> it = type.getParts(); it.hasNext();) {
			addPotentialPartTypes(list, it.next(), new ArrayList<ReferenceItem>(), type);
		}
		
		if (isImplementation) {
			ArrayList<ReferenceItem> path = new ArrayList<ReferenceItem>();
		//	path.add(new ReferenceItem("component-root"));
			for (Iterator<ComposableType> it = getComponentRoot().getPartTypes().iterator(); it.hasNext();) {
				list.add(new HasPartOfTypeExpression(it.next(), new StructureReference(getComponentRoot(), path, null, getComponentRoot()), false));
			}
			
			for (Iterator<PartDefinition> it = getComponentRoot().getParts(); it.hasNext();) {
				addPotentialPartTypes(list, it.next(), path, getComponentRoot());
			}
		}	
		return list;
	}


	private void addPotentialPartTypes(ArrayList<HasPartOfTypeExpression> list, PartDefinition definition, ArrayList<ReferenceItem> path, ComposableType root) {		
		for (Iterator<ComposableType> it = definition.getPossibleTypes(); it.hasNext();) {
			ComposableType ct = it.next();	
			ArrayList<ReferenceItem> newPath = new ArrayList<ReferenceItem>(path); 
			newPath.add(new ReferenceItem(definition.getName()));
			for (Iterator<ComposableType> i = ct.getPartTypes().iterator(); i.hasNext();) {
				list.add(new HasPartOfTypeExpression(i.next(), new StructureReference(root, newPath, null, null), false));
			}
		
			for (Iterator<PartDefinition> i = ct.getParts(); i.hasNext();) {
				addPotentialPartTypes(list, i.next(), newPath, root);
			}		
		}				
	}


	public ComponentTypeController getComponentRootController() {
		return (ComponentTypeController) controllerFactory.createController(getComponentRoot());
	}


	public ComponentType getComponentType(String name) {
		return model.getComponentType(name);
	}


	public void updateTypeName(ElementaryType type, String newName) {
		if (type instanceof FeatureType)
			model.updateTypeName((FeatureType)type, newName);
		else if (type instanceof ComponentType)
			model.updateTypeName((ComponentType)type, newName);
		else if (type instanceof InterfaceType)
			model.updateTypeName((InterfaceType)type, newName);
		else if (type instanceof AttributeValueType)
			model.updateTypeName((AttributeValueType)type, newName);
		modified();
	}


	private void modified() {
		modified(new ModelChangeEvent());
		
	}


	public void update() {
		notify(new ModelChangeEvent());
		
	}
}


