/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.model.exception.OnesOwnPartException;
import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.KumbangModellerPlugin;
import kumbang.modeller.ui.preferences.PreferencePage;
import kumbang.modeller.ui.properties.PartDefinitionPropertySource;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.views.properties.IPropertySource;

public class PartDefinitionController extends BasicController implements IPropertyChangeListener {

	private PartDefinition definition;
	private ComposableTypeController parent;
	private boolean isSimpleView;

	public PartDefinitionController(ModelController modelController, PartDefinition definition) {
		super(modelController);
		this.definition = definition;
		isSimpleView = KumbangModellerPlugin.getDefault().getPreferenceStore().getBoolean(PreferencePage.SIMPLE_VIEW);
		KumbangModellerPlugin.getDefault().getPreferenceStore().addPropertyChangeListener(this);
	}

	public void setParent(ComposableTypeController parent) {
		this.parent = parent;
	}
	
	public Object[] getChildren() {
		
		if (!isSimpleView || definition.getPossibleTypesCount() > 1) {
			Object[] types = new Object[definition.typeCount()];
			Iterator iterator = definition.getPossibleTypes();
			for (int i = 0; i < types.length; i++) {
				types[i] = modelController.getControllerFactory().createController(iterator.next());
				}
			
			return types;
		}	else return modelController.getControllerFactory().createController(definition.getPossibleTypes().next()).getChildren();
	}

	public boolean hasChildren() {
		if (!isSimpleView || definition.getPossibleTypesCount() > 1)
			return true;
		return modelController.getControllerFactory().createController(definition.getPossibleTypes().next()).hasChildren();
	
	}

	public String getName() {
		return definition.getName();
	}

	public int getType() {
		return PART_DEFINITION;
	}

	public String getDisplayText() {
		
	
		StringBuffer sb = new StringBuffer();
		if (isSimpleView && definition.getPossibleTypesCount() == 1)
			sb.append(((ComposableType)definition.getPossibleTypes().next()).getName());
		else 
			sb.append(definition.getName());
		
		
		sb.append(" [" + definition.getLowerBound() + "-" + definition.getUpperBound() + "]"); 

		if (definition.getSimilarity() == PartDefinition.simDifferent)
			sb.append( " {different}");
		else if (definition.getSimilarity() == PartDefinition.simSame)
			sb.append( " {same}");
		return sb.toString();
	}

	public int getLowerBound() {
		return definition.getLowerBound();
	}

	public int getUpperBound() {
		return definition.getUpperBound();
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new PartDefinitionPropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public void setLowerBound(String value) {
		try {
			int newValue = Integer.parseInt(value);
			definition.setLowerBound(newValue);
			modified();
		} catch (NumberFormatException nfe) {
			//XXX Throw something here
		}
	}
	
	public void setUpperBound(String value) {
		try {
			int newValue = Integer.parseInt(value);
			definition.setUpperBound(newValue);
			modified();
		} catch (NumberFormatException nfe) {
			//XXX Throw something here
		}

	}

	public String getImageString() {
		if (isSimpleView && definition.getPossibleTypesCount() == 1) {
			if (parent.getType() == COMPONENT_TYPE)
				return COMPONENT_TYPE_IMAGE_STRING;
			else 
				return FEATURE_TYPE_IMAGE_STRING;
		}
		
		if (parent.getType() == COMPONENT_TYPE)
			return PART_DEFINITION_IMAGE_STRING;
		else 
			return SUBFEATURE_IMAGE_STRING;
	}

	public void delete() {
		parent.removePartDefinition(definition.getName(), false);
		
		
	}

	public void addType(ComposableType composableType) {
		definition.addPossibleType(composableType);
		modified();
	}

	public void removeType(ComposableType composableType) {
		definition.removePossibleType(composableType);
		
	}

	public String getBackgroundString() {
		if (isSimpleView && definition.getPossibleTypesCount() == 1) {
			if (parent.getType() == COMPONENT_TYPE)
				return COMPONENT_TYPE_COLOR_STRING;
			else 
				return FEATURE_TYPE_COLOR_STRING;
		}
		
		
		if (parent.getType() == COMPONENT_TYPE)
			return PART_DEFINITION_COLOR_STRING;
		else 
			return SUBFEATURE_DEFINITION_COLOR_STRING;
	}

	public boolean isFeature() {
		return parent.getType() == FEATURE_TYPE;
	}

	public String[] getPotentialTypes() {
		Iterator it = definition.getPossibleTypes();
		LinkedList list = new LinkedList();
		while(it.hasNext()) {
			list.add(it.next());
		}
		String[] string = new String[list.size()];
		for (int i = 0; i < list.size(); i++)
			string[i] = ((ComposableType)list.get(i)).getName();
		return string;
	}

	
	public void setName(String text) {
		if (getName().equals(text))
			return;
		definition.setName(text);
		modified();
		
	}

	public void setLowerBound(int selection) {
		if (definition.getLowerBound() == selection)
			return;
		definition.setLowerBound(selection);
		modified();
		
	}

	public void setUpperBound(int selection) {
		if (definition.getUpperBound() == selection)
			return;
		definition.setUpperBound(selection);
		modified();
	}

	public void setPossibleTypes(String[] items) {
		List types = new ArrayList(items.length);
		for (int i = 0; i < items.length; i++)
			types.add(modelController.getComposableType(items[i]));	
		definition.setPossibleTypes(types);
		modified();
		
	}

	public int getSimilarity() {
		return definition.getSimilarity();
	}

	public void setSimilarity(int similarity) {
		if (getSimilarity() == similarity)
			return;
		definition.setSimilarity(similarity);
		modified();
		
	}
	
	public String getDisplayName() {	
		return definition.getDisplayName();
	}

	public void setDisplayName(String text) {
		definition.setDisplayName(text);
		modified();
	}

	public String getParentName() {
		return parent.getName();
	}
	
	public int getParentType() {
		return parent.getType();
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty() == PreferencePage.SIMPLE_VIEW) {
			isSimpleView = (Boolean)event.getNewValue();
			modelController.update();
		}
		
	}
}
