/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ConnectionController;
import kumbang.modeller.controllers.ModelController;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;



public class ConnectionDialog extends UndoableOperationDialog {
	
	private ComponentTypeController componentController;
	private Combo sourceCombo;
	private Combo targetCombo;
	
	private ConnectionController connectionController;
	
	private List<StructureReference> refs;
	private List<StructureReference> allRefs;
	private List<StructureReference> potentials = new ArrayList<StructureReference>();
	private ModelController modelController;
	protected StructureReference sourceRef;

	public ConnectionDialog(Shell shell, IUndoContext undoContext,  ComponentTypeController controller, ModelController modelController) {
		super(shell, undoContext, "Add Connection");
		this.modelController = modelController;
		this.componentController = controller;
	}

	public ConnectionDialog(Shell shell, IUndoContext undoContext,   ConnectionController connectionController, ModelController modelController) {
		super(shell, undoContext, "Connection Properties");
		this.connectionController = connectionController;
		this.modelController = modelController;
		this.componentController = connectionController.getParent();
		updateStatus(Status.OK_STATUS);
	}
	
	protected Control createDialogArea(Composite parent) {
		Composite composite =(Composite)super.createDialogArea(parent);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());
	
		new Label(composite, SWT.NULL).setText("Connect interface");

		refs = modelController.getAllInterfaces((ComponentType) componentController.getComposableType());
		allRefs = modelController.getAllInterfaces(modelController.getComponentRoot());
		sourceCombo = new Combo(composite, SWT.READ_ONLY);
		sourceCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());

		for (Iterator<StructureReference> it = refs.iterator(); it.hasNext();) {
			sourceCombo.add(it.next().getModelString(false));
		}
		
		sourceCombo.addSelectionListener(new SelectionAdapter() {
			
					public void widgetSelected(SelectionEvent e) {	
							updateTarget();
					}
		});
		

		new Label(composite, SWT.NULL).setText("to interface");
		
		targetCombo = new Combo(composite, SWT.READ_ONLY);
		targetCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		targetCombo.addSelectionListener(new SelectionAdapter(){
			
			public void widgetSelected(SelectionEvent e) {	
				checkStatus();
			}
			
		});		
	
		if (connectionController != null) {
			for (int i = 0; i < sourceCombo.getItemCount(); i++) {
				if (sourceCombo.getItem(i).equals(connectionController.getSourceString())) {
					sourceCombo.select(i);
					break;
				}
			}
			
			updateTarget();
			
			for (int i = 0; i < targetCombo.getItemCount(); i++) {
				if (targetCombo.getItem(i).equals(connectionController.getTargetString())) {
					targetCombo.select(i);
					break;
				}
			}
			checkStatus();
		}
		
		
		return composite; 
		
	}
	
	
	
	protected void updateTarget() {
		updateStatus(status);
		potentials.clear();	
		targetCombo.removeAll();
		sourceRef = refs.get(sourceCombo.getSelectionIndex());	
			
		for (StructureReference ref: allRefs) {
			if (canConnect(sourceRef, ref)) {
				potentials.add(ref);
				targetCombo.add(ref.getModelString(false));
			}
			
		}
			
		
	}

	@SuppressWarnings("unchecked")
	private boolean canConnect(StructureReference source, StructureReference target) {
			
		if (source == target) 
			return false;
		
		
		InterfaceDefinition sourceInterface = modelController.getInterfaceDefinition(source);
		InterfaceDefinition targetInterface = modelController.getInterfaceDefinition(target);
	
		if (sourceInterface.equals(targetInterface))
			return false;
		
		if (sourceInterface == null)
			sourceInterface = componentController.getInterface(source.getInterfaceName());
		if (targetInterface == null)
			targetInterface = componentController.getInterface(target.getInterfaceName());
		
		if (sourceInterface != null && targetInterface!= null) {
		
				
			List<InterfaceType> sourceInterfaceTypeList = sourceInterface.getPossibleTypesList();
			List<InterfaceType> targetInterfaceTypeList = targetInterface.getPossibleTypesList();
			for (int i = 0; i < sourceInterfaceTypeList.size() && i < targetInterfaceTypeList.size(); i++){		
				if (sourceInterfaceTypeList.get(i).isDisjointFrom((targetInterfaceTypeList.get(i)))) {
					return false;
				}
			}		
		}	
		return true;
	
	}



	protected void checkStatus() {
		updateStatus(Status.OK_STATUS);

	}
	


	protected void updateInfo() throws PropertyAlreadyDefinedException {
		componentController.removeConnection(connectionController);	

		InterfaceDefinition sourceInterface = modelController.getInterfaceDefinition(sourceRef);
		InterfaceDefinition targetInterface = modelController.getInterfaceDefinition(potentials.get(targetCombo.getSelectionIndex()));
	
		PartDefinition sourcePart = modelController.getPartDefinition(sourceRef);
		PartDefinition targetPart = modelController.getPartDefinition(potentials.get(targetCombo.getSelectionIndex()));
		
		componentController.addConnection(sourceInterface, sourcePart, targetInterface, targetPart);
		
	}

	protected void doNew() throws PropertyAlreadyDefinedException {
		
		InterfaceDefinition sourceInterface = modelController.getInterfaceDefinition(sourceRef);
		InterfaceDefinition targetInterface = modelController.getInterfaceDefinition(potentials.get(targetCombo.getSelectionIndex()));
		
		if (sourceInterface == null)
			sourceInterface = componentController.getInterface(sourceRef.getInterfaceName());
		if (targetInterface == null)
			targetInterface = componentController.getInterface(potentials.get(targetCombo.getSelectionIndex()).getInterfaceName());
	
		PartDefinition sourcePart = modelController.getPartDefinition(sourceRef);
		PartDefinition targetPart = modelController.getPartDefinition(potentials.get(targetCombo.getSelectionIndex()));
		componentController.addConnection(sourceInterface, sourcePart, targetInterface, targetPart);
	}

	protected boolean isPropertyDialog() {
		return connectionController != null;
	}

}
