/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.ArrayList;
import java.util.Iterator;

import kumbang.core.model.constraint.AttributeExpression;
import kumbang.core.model.constraint.BinaryExpression;
import kumbang.core.model.constraint.BooleanExpression;
import kumbang.core.model.constraint.CardinalityExpression;
import kumbang.core.model.constraint.ComparisonExpression;
import kumbang.core.model.constraint.ConjunctiveExpression;
import kumbang.core.model.constraint.ConnectedExpression;
import kumbang.core.model.constraint.EquivalenceExpression;
import kumbang.core.model.constraint.Expression;
import kumbang.core.model.constraint.HasInstancesExpression;
import kumbang.core.model.constraint.HasPartOfTypeExpression;
import kumbang.core.model.constraint.ImplicationExpression;
import kumbang.core.model.constraint.InstanceOfExpression;
import kumbang.core.model.constraint.PresentExpression;
import kumbang.core.model.constraint.QuantifiedExpression;
import kumbang.core.model.constraint.ReferenceExpression;
import kumbang.core.model.type.ComposableType;
import kumbang.modeller.KumbangModellerPlugin;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ConnectionController;
import kumbang.modeller.controllers.ConstraintController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;
import kumbang.modeller.ui.dialogs.UndoableOperationDialog;
import kumbang.modeller.ui.editor.operations.AddConstraintOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

public class ConstraintDialog extends UndoableOperationDialog  /* implements ModifyListener */ {
	
	private final static String AND_OPERATOR = "and";
	private final static String OR_OPERATOR= "or";
	private final static String EQUIVALENCE_OPERATOR = "<=>";
	private final static String IMPLICATION_OPERATOR = "=>";
	
	
	private final static String[] OPERATIONS = new String[]{AND_OPERATOR, OR_OPERATOR, EQUIVALENCE_OPERATOR, IMPLICATION_OPERATOR};	
	
	private class AddOperatorDialog extends StatusDialog {

		private Combo ops;
		private String op;

		
		public AddOperatorDialog(Shell parent) {
			super(parent);
			
		}
		

		public AddOperatorDialog(Shell shell, String selection) {
			this(shell);
			this.op = selection;
		}


		protected Control createDialogArea(Composite parent) {
			updateStatus(new Status(Status.ERROR, KumbangModellerPlugin.PLUGIN_ID, Status.ERROR,"",null));
			
			Composite composite = new Composite(parent, SWT.NULL);
			composite.setLayout(new GridLayout(1, true));
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			
			new Label(composite, SWT.NULL).setText("Choose Operator:");
			
			ops = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);
			ops.setItems(OPERATIONS);
			ops.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
			ops.addSelectionListener(new SelectionAdapter(){
				
				public void widgetSelected(SelectionEvent e) {
					updateStatus(Status.OK_STATUS);
				}

			});
			if (op != null)
				for (int i = 0; i < ops.getItemCount(); i++) {
					if (ops.getItem(i).equals(op)) {
						ops.select(i);
						break;
					}
				}
			
			return composite;
		}
		
		protected void okPressed() {
			if (op == null)
				addOperator(ops.getSelectionIndex()); 
			else 
				setOperator(ops.getSelectionIndex());
			super.okPressed();
		}
	}	
	
	private ModelController modelController;

	private List conditions;

	private ArrayList<BooleanExpression> condList = new ArrayList<BooleanExpression>();
	private ArrayList<String> operators = new ArrayList<String>();
	private Button addOperator;
	private Button addExpression;
	private boolean isImplementation;
	private ConstraintController controller;
	protected ComposableTypeController composableController;
	private boolean changed = false;
	private Button edit;
	private Button delete;

	public ConstraintDialog(Shell shell, IUndoContext undoContext, ModelController modelController, boolean isImplementation, ComposableTypeController composableController) {
		super(shell, undoContext, "Add Constraint");
		this.isImplementation = isImplementation;
		this.modelController = modelController;
		this.composableController = composableController;
	}
	
	public ConstraintDialog(Shell shell, IUndoContext undoContext, ModelController modelController, ConstraintController controller) {
		super(shell, undoContext, "Edit Constraint");
		this.isImplementation = ConstraintController.isImplementation(controller.getExpression());
		this.modelController = modelController;
		this.composableController = controller.getParentController();
		this.controller = controller;
	}

	public void addOperator(int selectionIndex) {
	

		
		if (conditions.getSelectionIndex() < 0) {
			conditions.add(OPERATIONS[selectionIndex]);
			operators.add(OPERATIONS[selectionIndex]);
			
		}
		else { 
			conditions.add(OPERATIONS[selectionIndex], conditions.getSelectionIndex() + 1);
			operators.add(countOperatorIndex(), OPERATIONS[selectionIndex]);
		}
		changed = true;
		
	}
	
	public void setOperator(int selectionIndex) {
			
		operators.set(countOperatorIndex(),OPERATIONS[selectionIndex]);
		conditions.setItem(conditions.getSelectionIndex(), OPERATIONS[selectionIndex]);
		changed = true;
		
	}
	

	public void addCondition(BooleanExpression expression, boolean update) {
	
		if (update) {
			conditions.setItem(conditions.getSelectionIndex(), expression.getModelString(isImplementation));
			condList.set(countExpressionIndex(), expression);
			return;
		}
		
		if (conditions.getSelectionIndex() < 0) {
			conditions.add(expression.getModelString(isImplementation));
			condList.add(expression);
		}
		else {
			conditions.add(expression.getModelString(isImplementation), conditions.getSelectionIndex() + 1);
			condList.add(countExpressionIndex(), expression);
		}
		changed = true;
	}
	
	private int countOperatorIndex() {
		int index = 0;
		for (int i = 0; i < conditions.getSelectionIndex(); i++)
			if (isOperator(conditions.getItem(i)))
				index++;
		
		return index;
	}
	
	private int countExpressionIndex() {
		int index = 0;
		for (int i = 0; i < conditions.getSelectionIndex(); i++)
			if (!isOperator(conditions.getItem(i)))
				index++;
		
		return index;
	}

	protected Control createDialogArea(Composite parent) {
		Composite composite =(Composite)super.createDialogArea(parent);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout(1, true));
		
		new Label(composite, SWT.NULL).setText("Current constraint:");
		composite.setLayout(new GridLayout(1, true));
		conditions = new List(composite, SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.CENTER);
		conditions.setLayoutData(LayoutFactory.getInstance().getFillGridData());
	
		conditions.addSelectionListener(new SelectionAdapter(){
			
			public void widgetSelected(SelectionEvent e) {
				checkStatus();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				editCurrent();
				
			}

		});
		
		Composite buttons = new Composite(composite, SWT.NULL);
		buttons.setLayout(new GridLayout(2, true));
		buttons.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		
		addExpression = new Button(buttons, SWT.NULL);
		addExpression.setText("Add Expression...");
		addExpression.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		addExpression.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				new ChooseExpressionTypeDialog(ConstraintDialog.this, composableController, getShell(), isImplementation, modelController).open();

				checkStatus();
			}

		});
		
		addOperator = new Button(buttons, SWT.NULL);
		addOperator.setText("Add Operator...");
		addOperator.setEnabled(false);
		addOperator.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		addOperator.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				
				new AddOperatorDialog(getShell()).open();
				checkStatus();
			}

		});
//		
		
		
		edit = new Button(buttons, SWT.NULL);
		edit.setText("Edit...");
		edit.setEnabled(false);
		edit.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		edit.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				
				editCurrent();
			}

		});
		
		delete = new Button(buttons, SWT.NULL);
		delete.setText("Delete");
		delete.setEnabled(false);
		delete.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		delete.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				
				String selection = conditions.getItem(conditions.getSelectionIndex());
				if (isOperator(selection)) {
					operators.remove(countOperatorIndex());
				} else {
					condList.remove(countExpressionIndex());
				}
				conditions.remove(conditions.getSelectionIndex());
				
				checkStatus();
			}

		});
		
		if (controller != null) {
			addCurrentExpression(controller.getExpression());
			checkStatus();
		}
//		Button modify = new Button(buttons, SWT.NULL);
//		modify.setText("Modify...");
//		modify.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
//		Button remove = new Button(buttons, SWT.NULL);
//		remove.setText("Remove Last");
//		remove.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		return composite; 
	}
	

	
	
	
	protected void editCurrent() {
		
		String selection = conditions.getItem(conditions.getSelectionIndex());
		if (isOperator(selection))
			new AddOperatorDialog(getShell(), selection).open();
		else {
			ConstraintExpressionDialog ced = getExpressionDialog(condList.get(countExpressionIndex()));
			if (ced != null)
				ced.open();
			
		}
		checkStatus();
		

		
	}
	//XXX Finish this to edit expressions!
	protected ConstraintExpressionDialog getExpressionDialog(BooleanExpression expression) {
		if (expression instanceof ComparisonExpression) {
				
			if (((ComparisonExpression)expression).getLeft() instanceof CardinalityExpression)
				return new CardinalityExpressionDialog(this, composableController, getShell(), isImplementation, modelController, (ComparisonExpression)expression);	
			return new AttributeExpressionDialog(this, composableController, getShell(), isImplementation, modelController, (ComparisonExpression)expression);
		}
		
		if (expression instanceof PresentExpression)
			return new PresentExpressionDialog(this, composableController, getShell(), isImplementation, modelController,  (PresentExpression)expression);
		
		if (expression instanceof InstanceOfExpression)
			return new InstanceOfExpressionDialog(this, composableController, getShell(), isImplementation, modelController,  (InstanceOfExpression)expression);
		
		if (expression instanceof HasPartOfTypeExpression)
			return new HasPartOfTypeExpressionDialog(this, composableController, getShell(), isImplementation, modelController,  (HasPartOfTypeExpression)expression);
		
		if (expression instanceof HasInstancesExpression) 
			return new HasInstancesExpressionDialog(this, composableController, getShell(), isImplementation, modelController,  (HasInstancesExpression)expression);
			
		if (expression instanceof ConnectedExpression)
			return new ConnectedExpressionDialog(this, composableController, getShell(), isImplementation, modelController,  (ConnectedExpression)expression);
			
		
		return null;
	}

	protected void checkStatus() {
	
	
		if (conditions.getSelectionIndex() >=0) {				
			String selection = conditions.getItem(conditions.getSelectionIndex());
			
			if (isOperator(selection)) {
				addOperator.setEnabled(false);
				addExpression.setEnabled(true);
			} else {
				addOperator.setEnabled(true);
				addExpression.setEnabled(false);
			}
			
			edit.setEnabled(true);
			delete.setEnabled(true);
		} else {
			edit.setEnabled(false);
			delete.setEnabled(false);
			addOperator.setEnabled(!expressionNext());
			addExpression.setEnabled(expressionNext());
		}
		isComplete();
	}
	
	private boolean expressionNext() {

		if (conditions.getItemCount() == 0)
			return true;
		
		return isOperator(conditions.getItem(conditions.getItemCount() - 1));
	}

	private void isComplete() {
		if (expressionNext()) {
			updateStatus(status);
			return;
		}
		for (int i = 0; i < conditions.getItemCount(); i++) {
			if ( i % 2 == 0 && isOperator(conditions.getItem(i)) ||
				i % 2 != 0 && !isOperator(conditions.getItem(i))) {
				updateStatus(status);
				return;
			} 
			
		}
		
		updateStatus(Status.OK_STATUS);
		
	}

	private boolean isOperator(String s) {
		return operators.contains(s);
	}

	protected void updateInfo() {
		if (changed) {
			composableController.removeConstraint(controller.getExpression());
			composableController.addConstraint(getExpression());
		}
	}

	protected void doNew() throws ExecutionException  {
		
		doOperation(new AddConstraintOperation(getExpression(), composableController));

		
	}

	private BooleanExpression getExpression() {
		
		ArrayList<BooleanExpression> oldCond = condList;
		ArrayList<String> oldOp = operators;
		combineConjunctiveOperations();
		
		if (condList.size() == 1 && operators.size() ==0) {
			return condList.get(0);
			
		} else if (condList.size() == 2 && operators.size() == 1) {
			String op = operators.get(0);			
			if (op.equals(IMPLICATION_OPERATOR)) {
				BooleanExpression be = new ImplicationExpression(condList.get(0), condList.get(1));
				condList = oldCond;
				operators = oldOp;
				return be;
				
			} else if (op.equals(EQUIVALENCE_OPERATOR)) {
				BooleanExpression be = new EquivalenceExpression(condList.get(0), condList.get(1));
				condList = oldCond;
				operators = oldOp;
				return be;
			} 
	
		}
		condList = oldCond;
		operators = oldOp;
		
		return null;
	}

	private void combineConjunctiveOperations() {
		
		combineAndOperations();
		combineOrOperations();
	}
	
	private void combineOrOperations() {
		Iterator<BooleanExpression> iterator = condList.iterator();
		ArrayList<BooleanExpression> newCondList = new ArrayList<BooleanExpression>();
		ArrayList<String> newOperatorList = new ArrayList<String>();
		ConjunctiveExpression ce = null;
		
		for (int i = 0; i < operators.size(); i++) {
			String op = operators.get(i);
			if (op.equals(OR_OPERATOR)) {
				if (ce == null) {
					ce = new ConjunctiveExpression(ConjunctiveExpression.opOr);
					ce.addPrimitive(iterator.next());
				}
				ce.addPrimitive(iterator.next());
			} else {
				if (ce != null) {
					newCondList.add(ce);
					newCondList.add(iterator.next());
					ce = null;
					newOperatorList.add(op);
				} else {
					newCondList.add(iterator.next());
					newOperatorList.add(op);
				}
			}
		}
		
		if (ce != null)
			newCondList.add(ce);
		while(iterator.hasNext())
			newCondList.add(iterator.next());
		condList = newCondList;
		operators = newOperatorList;
		
	}

	private void combineAndOperations() {
		Iterator<BooleanExpression> iterator = condList.iterator();
		ArrayList<BooleanExpression> newCondList = new ArrayList<BooleanExpression>();
		ArrayList<String> newOperatorList = new ArrayList<String>();
		ConjunctiveExpression ce = null;
		
		for (int i = 0; i < operators.size(); i++) {
			String op = operators.get(i);
			if (op.equals(AND_OPERATOR)) {
				if (ce == null) {
					ce = new ConjunctiveExpression(ConjunctiveExpression.opAnd);
					ce.addPrimitive(iterator.next());
				}
				ce.addPrimitive(iterator.next());
			} else {
				if (ce != null) {
					newCondList.add(ce);
					newCondList.add(iterator.next());
					ce = null;
					newOperatorList.add(op);
				} else {
					newCondList.add(iterator.next());
					newOperatorList.add(op);
				}
			}
		}
		
		if (ce != null)
			newCondList.add(ce);
		while (iterator.hasNext())
			newCondList.add(iterator.next());
		condList = newCondList;
		operators = newOperatorList;
	}

	private void addCurrentExpression(Expression expression) {
		
		if (expression instanceof ConjunctiveExpression) {
			ConjunctiveExpression cje = (ConjunctiveExpression)expression;
			for (Iterator<Expression> it = cje.getPrimitives(); it.hasNext();) {
				addCurrentExpression(it.next());
				if (it.hasNext()) 
					addOperator(cje.getOperator() == ConjunctiveExpression.opAnd ? AND_OPERATOR : OR_OPERATOR);
			
			}
			return;
		}
		
		if (expression instanceof EquivalenceExpression) {
			EquivalenceExpression ee = (EquivalenceExpression)expression;
			addCurrentExpression(ee.getLeft());
			addOperator(EQUIVALENCE_OPERATOR);
			addCurrentExpression(ee.getRight());
			return;
		}	
		
			
		if (expression instanceof ImplicationExpression) {
			ImplicationExpression ie = (ImplicationExpression)expression;
			
			addCurrentExpression(ie.getTail());
			addOperator(IMPLICATION_OPERATOR);
			addCurrentExpression(ie.getHead());
			
			return;
		}
		
		addCondition((BooleanExpression)expression, false);
	}
	

	
	private void addOperator(String string) {
		operators.add(string);
		conditions.add(string);
		
	}

	protected boolean isPropertyDialog() {
		return controller != null;
	}

}
