/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dnd;

import kumbang.core.model.exception.OnesOwnPartException;
import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.InterfaceDefinitionController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.controllers.PartDefinitionController;
import kumbang.modeller.ui.dialogs.AttributeDefinitionDialog;
import kumbang.modeller.ui.dialogs.InterfaceDefinitionDialog;
import kumbang.modeller.ui.dialogs.PartDefinitionDialog;
import kumbang.modeller.ui.editor.operations.AddTypesToInterfaceDefinitionOperation;
import kumbang.modeller.ui.editor.operations.AddTypesToPartDefinitionOperation;
import kumbang.modeller.ui.editor.operations.SetFeatureRootOperation;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.TransferData;

public class FeatureViewerDropAdapter extends KumbangModellerDropAdapter {

	


	public FeatureViewerDropAdapter(TreeViewer viewer, ModelController controller, IUndoContext undoContext) {
		super(viewer, controller, undoContext);
	}

	public boolean performDrop(Object data) {
		
		if (data instanceof TypeTransferData) {
		
			TypeTransferData kData = (TypeTransferData)data;
		
		
			// Setting the Feature root
			if (!modelController.isFeatureRootSet() && kData.getType() == FEATURE_TYPE) {
				return doOperation(new SetFeatureRootOperation(modelController, kData.getName()));
			}
		
			if (getCurrentTarget() == null || !(getCurrentTarget() instanceof BasicController))
				return false;
		
			BasicController target = (BasicController) getCurrentTarget();
		
			switch(kData.getType()) {
	
			case FEATURE_TYPE:		
				if (target.getType() == PART_DEFINITION) {
					return doOperation( new AddTypesToPartDefinitionOperation(modelController, (PartDefinitionController)target, kData.getName()));
				}
			
				if (target.getType() == FEATURE_TYPE) {
					new PartDefinitionDialog(undoContext, modelController, (ComposableTypeController) target, kData.getName()).open();
					return true;
				}
				return false;
			
			case ATTRIBUTE_VALUE_TYPE:
				if (target.getType() == FEATURE_TYPE) {
					new AttributeDefinitionDialog(undoContext, modelController, (ComposableTypeController) target, kData.getName()).open();
					return true;
				}
	
				return false;
			
			}	
		}
		
		if (data instanceof DefinitionTransferData) {
			if (getCurrentTarget() == null || !(getCurrentTarget() instanceof FeatureTypeController))
				return false;
			
			DefinitionTransferData dData = (DefinitionTransferData)data;
			FeatureTypeController target = (FeatureTypeController) getCurrentTarget();
			if (dData.getParentType() !=  FEATURE_TYPE)
				return false;
			
			FeatureType exParent = modelController.getFeatureType(dData.getParentName());
			if (dData.getType() == PART_DEFINITION) {
				PartDefinition d = exParent.getPart(dData.getName());
				
				try {
					target.addDefinition(d);
				} catch (PropertyAlreadyDefinedException e) {
					MessageDialog.openError(null, "Drop Failed", e.getMessage());
					return false;
				} catch (OnesOwnPartException e) {
					MessageDialog.openError(null, "Drop Failed", e.getMessage());
					return false;
				}
				exParent.removePart(d);
				return true;
				
			} else if (dData.getType() == ATTRIBUTE_DEFINITION) {
				AttributeDefinition d = exParent.getAttributeDefinition(dData.getName());
				try {
					target.addAttributeDefinition(new Name(d.getName(), d.getDisplayName()), d.getType().getName());
				} catch (Exception e) {
					MessageDialog.openError(null, "Drop Failed", e.getMessage());
					return false;
				}
				
				exParent.removeAttribute(d);
				return true;
				
				
			}
			
		
		}
		return false;
		
	}

	public boolean validateDrop(Object target, int operation, TransferData transferType) {	
		
		// Only a Feature type can be dragged to an empty Feature part structure
		if (target == null)
			return FeatureTypeTransfer.getInstance().isSupportedType(transferType);
		
		// This should never happen, but better be safe
		if (!(target instanceof BasicController))
			return false;
		
	
		BasicController element = (BasicController) target;
	
		// Making a new definition
		if (element.getType() == FEATURE_TYPE)
			return FeatureTypeTransfer.getInstance().isSupportedType(transferType) 
			|| AttributeTypeTransfer.getInstance().isSupportedType(transferType)
			|| PartDefinitionTransfer.getInstance().isSupportedType(transferType)
			|| AttributeDefinitionTransfer.getInstance().isSupportedType(transferType); 
		
		// Adding types to an existing definition
		
		if (element.getType() ==  PART_DEFINITION)
			return FeatureTypeTransfer.getInstance().isSupportedType(transferType);;
		
		if (element.getType() == ATTRIBUTE_DEFINITION)
			return false;
		
		return false;
	} 
    
  
	
}
