/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.editor;

import java.io.IOException;

import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.core.model.core.KumbangModel;
import kumbang.core.model.exception.ModelException;
import kumbang.modeller.KumbangModellerPlugin;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.event.ModelChangeEvent;
import kumbang.modeller.ui.event.ModelChangeListener;
import kumbang.modeller.ui.event.ModelChangeProvider;
import kumbang.modeller.ui.event.ModelTypeChangedEvent;
import kumbang.modeller.ui.event.ModellerEvent;
import kumbang.modeller.ui.viewers.KumbangLabelProvider;
import kumbang.modeller.ui.viewers.ViewerActionFactory;
import kumbang.modeller.util.Constants;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.ObjectUndoContext;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IPerspectiveRegistry;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.editors.text.ILocationProvider;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.part.EditorPart;


/**
 * KumbangModllerEditor class extends EditorPart, making it an Eclipse editor with
 * all associated functionality. 
 * 
 * The class handles the editor area, where a name and type of the model can be changed.
 * 
*/
public class KumbangModellerEditor extends EditorPart implements Constants, ModelChangeListener, ModelChangeProvider {

	public static final String EDITOR_ID = "kumbang.modeller.editor";

	private static final String objectSaveQuestion = "Model cannot be saved as a text file because it is not a valid model (root type(s) are not set).\n" + 
													"Save as a serialized Java object instead?\n\n" +
													"You may not use the saved model object outside  of this version of Kumbang Modeller, but you can save it again as a text file after roots are set.";

	private static final String STRING_SAVEDIALOG_TITLE = "Select the location and the filename of the exported model.";
	private FormToolkit toolkit;
	private ModelController modelController;

	private IUndoContext undoContext = new ObjectUndoContext(this);
	private ViewerActionFactory actionFactory;
	private KumbangLabelProvider labelProvider;
	
	private ListenerList listeners = new ListenerList();
	private Text nameText;
	private Form nameForm;
	private boolean parsingFailed;

	private Form form;
	private Combo modelTypeCombo;

	
	
	public void doSave(IProgressMonitor monitor) {
	
		if (!modelController.canBeSavedAsText()) {
			if(MessageDialog.openQuestion(getSite().getShell(), "Confirm save" , objectSaveQuestion)) 
					modelController.saveAsObject();
			
			return;
		}
		modelController.save();
		updateDirtyState();
		
	}

	public void doSaveAs() {
		boolean asObject = false;
		if (!modelController.canBeSavedAsText()) {
			asObject = MessageDialog.openQuestion(getSite().getShell(), "Confirm save", objectSaveQuestion);
			if (!asObject)
				return;
		}
		FileDialog fd = new FileDialog(getSite().getShell(), SWT.SAVE);
		fd.setFilterExtensions(new String[] {"*.kbm", "*.*"});	
		String file = fd.open();
		modelController.saveAs(file, false);
		updateDirtyState();

	}

	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		modelController = new ModelController();
		modelController.addModelChangeListener(this);   
		labelProvider = KumbangLabelProvider.getInstance();		
		actionFactory = new ViewerActionFactory(undoContext, modelController, getSite().getShell());
	}

	public boolean isDirty() {
		return modelController.isModified();
	}

	public boolean isSaveAsAllowed() {
		return true;
	}
	
	public void createPartControl(Composite parent) {
		
	     IWorkbenchWindow window = getSite().getWorkbenchWindow();
	        if (window != null && window.getActivePage() != null && !window.getActivePage().getPerspective().getId().equals(KumbangModellerPlugin.PERSPECTIVE_ID)) {
	             IPerspectiveRegistry reg = window.getWorkbench().getPerspectiveRegistry();
	             if (reg != null)
	            	 window.getActivePage().setPerspective(reg.findPerspectiveWithId(KumbangModellerPlugin.PERSPECTIVE_ID));
	        }
		
		toolkit = new FormToolkit(parent.getDisplay());
		
		
		
		
		new UndoRedoActionGroup(this.getSite(), undoContext, false).fillActionBars(getEditorSite().getActionBars());
		//getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.DELETE.getId(),
				//actionFactory.getDeleteAction());
		
		try {
			IEditorInput input = getEditorInput();
			
			if (input instanceof KumbangEditorInput) {
				modelController.createModel(input.getName(),((KumbangEditorInput)input).getType(),  ((KumbangEditorInput)input).getFile().getRawLocation());
				
			} else if (input instanceof IFileEditorInput) {
				IFileEditorInput fileEditorInput = (IFileEditorInput)input;			
				modelController.loadModel(fileEditorInput.getFile().getRawLocation());
		
			} else if (input instanceof ILocationProvider) {
				ILocationProvider path = (ILocationProvider)getEditorInput();
				modelController.loadModel(path.getPath(input));
			}
			
			setPartName(modelController.getFileName());
			
		} catch (ModelException e) {
			Form form = toolkit.createForm(parent);
			form.setText("Model could not be opened");
			form.getBody().setLayout(new GridLayout(1, true));
			toolkit.createLabel(form.getBody(), "Reason: " + e.getMessage());
	
			parsingFailed = true;
			setPartName(modelController.getFileName());
			//e.printStackTrace();
			return;
			
		} 
		
		// Sometimes views don't update otherwise
		notify(new ModelChangeEvent());
		form = toolkit.createForm(parent);
	
		form.setText(modelController.getModelType() + " model");
		form.getBody().setLayout(new GridLayout(3, true));
		
		nameForm = toolkit.createForm(form.getBody());
		nameForm.getBody().setLayout(new GridLayout(4, true));
		nameForm.getBody().setLayoutData(new GridData(GridData.FILL_BOTH));
		
		toolkit.createLabel(nameForm.getBody(), "Model name:");
		
		nameText = toolkit.createText(nameForm.getBody(), modelController.getName(), SWT.BORDER);
		nameText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		
		toolkit.createButton(nameForm.getBody(), "Change", SWT.NULL).addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent e) {
				modelController.setModelName(nameText.getText());
				setPartName(modelController.getFileName());
				updateDirtyState();
			}

		});
		
		Form typeForm = toolkit.createForm(form.getBody());
		typeForm.getBody().setLayout(new GridLayout(3, false));
		typeForm.getBody().setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		toolkit.createLabel(typeForm.getBody(), "Model type:");
		
		modelTypeCombo = new Combo(typeForm.getBody(), SWT.READ_ONLY);
		toolkit.adapt(modelTypeCombo);
		
		
		String[] modelTypes = new String[] {KumbangModel.MODEL_KIND , ForfamelModel.MODEL_KIND , KoalishModel.MODEL_KIND};
		modelTypeCombo.setItems(modelTypes);
		
		for (int i = 0; i <modelTypes.length; i++) {
			if (modelTypes[i].startsWith(modelController.getModelType())) {
				modelTypeCombo.select(i);
				break;
			}
		}

		toolkit.createButton(typeForm.getBody(), "Change", SWT.NULL).addSelectionListener(new SelectionAdapter(){
			
			public void widgetSelected(SelectionEvent event) {
				String selected = modelTypeCombo.getItem(modelTypeCombo.getSelectionIndex());
			
				if (selected.startsWith(modelController.getModelType()))
					return;
				
				String messageString = null;
				
				if (selected.startsWith(ForfamelModel.MODEL_KIND) && modelController.hasComponentModelData()) 
					messageString = "Changing model type to a Forfamel model causes the loss of data used only for Kumbang and Koalish models (e. g. component types and interfaces). Proceed?";
				else if (selected.startsWith(KoalishModel.MODEL_KIND) && modelController.hasFeatureModelData()) {
					messageString = "Changing model type to a Koalish model causes the loss of data used only for Kumbang and Forfamel models (e. g. feauture types). Proceed?";
				}
			
				if (messageString != null && ! MessageDialog.openConfirm(getSite().getShell(), "Confirm Model Type Change", messageString))
					return;
				
				
				modelController.setModelType(selected);
			}		
		});	
		
		toolkit.createLabel(typeForm.getBody(), "Misc operations:");
		
		// The export as uml button
		toolkit.createButton(typeForm.getBody(), "Export as UML2", SWT.NULL).addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String fileName = showFileDialog();
				if (fileName != null)
					exportAsUml(fileName);
			}
		});
		
		// The check model button
		toolkit.createButton(typeForm.getBody(), "Check model validity", SWT.NULL).addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				checkModel();
			}
		});
	}
	
	private void exportAsUml(String fileName) {
		try {
//			getSite().getPage().g
//			ResourcesPlugin.getWorkspace().getRoot().findM
			modelController.exportAsUml(fileName);
		} catch (ModelException e) {
			MessageDialog.openError(getSite().getShell(), "Model Exception when trying to export as UML", e.getMessage());
		}
	}

	private void checkModel() {
		try {
			if (modelController.checkModel())
				MessageDialog.openInformation(getSite().getShell(), "Model is valid", "Model is valid");
			else
				MessageDialog.openError(getSite().getShell(), "Model is inconsistend", "Model is inconsistent");
		} catch (ModelException e) {
			MessageDialog.openError(getSite().getShell(), "ModelException when checking the model", e.getMessage());
		} catch (IOException e) {
			MessageDialog.openError(getSite().getShell(), "IOException when checking the model", e.getMessage());
		}
	}



	
	private String showFileDialog() {
		FileDialog dlg = new FileDialog(getSite().getShell(), SWT.SAVE);
		dlg.setText(STRING_SAVEDIALOG_TITLE);
		dlg.setFilterExtensions(new String[] {"*.uml", "*.*"});
		return dlg.open();
	}
	
	public void setFocus() {
		// TODO Auto-generated method stub

	}


	public void updateDirtyState() {
		firePropertyChange(PROP_DIRTY);
	}

	public FormToolkit getToolkit() {
		return toolkit;
	}
	
	public ModelController getModelController() {
		return modelController;
	}

	public IUndoContext getUndoContext() {
		return undoContext;
	}

	public KumbangLabelProvider getLabelProvider() {
		return labelProvider;
	}

	public ViewerActionFactory getActionFactory() {
		return actionFactory;
	}

	public void modelChanged(ModellerEvent event) {
		//TODO: Update the change here, too
		updateDirtyState();
		
		notify(event);
	}

	public void addModelChangeListener(ModelChangeListener listener) {
		listeners.add(listener);
		
	}

	public void removeModelChangeListener(ModelChangeListener listener) {
		listeners.remove(listener); 
		
	}

	public boolean parsedOK() {
		return !parsingFailed;
	}
	
	public void notify(ModellerEvent event) {
		Object[] all = listeners.getListeners();
		
		for (int i = 0; i < all.length; i++) {
			if (all[i] instanceof ModelChangeListener)
				((ModelChangeListener)all[i]).modelChanged(event);
		}
		if (event instanceof ModelTypeChangedEvent) {
			form.setText(modelController.getModelType() + " model");
		}
	}


}
