/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;

import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.AttributeDefinitionDialog;
import kumbang.modeller.ui.dialogs.AttributeValueTypeDialog;
import kumbang.modeller.ui.dialogs.ComposableTypeDialog;
import kumbang.modeller.ui.dialogs.ConnectionDialog;
import kumbang.modeller.ui.dialogs.InterfaceDefinitionDialog;
import kumbang.modeller.ui.dialogs.InterfaceTypeDialog;
import kumbang.modeller.ui.dialogs.PartDefinitionDialog;
import kumbang.modeller.ui.dialogs.PropertyDialogFactory;
import kumbang.modeller.ui.dialogs.SelectComponentRootDialog;
import kumbang.modeller.ui.dialogs.SelectFeatureRootDialog;
import kumbang.modeller.ui.dialogs.constraints.ConstraintDialog;
import kumbang.modeller.ui.editor.operations.SetComponentRootOperation;
import kumbang.modeller.ui.editor.operations.SetFeatureRootOperation;
import kumbang.modeller.util.Constants;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Shell;

public class ViewerActionFactory {

	private final IUndoContext undoContext;
	private final ModelController modelController;
	private final Shell shell;
	
	private Action newInterfaceAction;
	private Action newAttributeTypeAction;
	private Action newFeatureTypeAction;
	private Action newComponentTypeAction;
	private Action selectComponentRootAction;
	private Action selectFeatureRootAction;
	private Action createComponentRootAction;
	private Action createFeatureRootAction;


	
	public ViewerActionFactory(IUndoContext undoContext, ModelController modelController, Shell shell) {
		this.undoContext = undoContext;
		this.modelController = modelController;
		this.shell = shell;
		
	}

	public Action getNewInterfaceAction() {
		
		if (newInterfaceAction == null) {
			newInterfaceAction  = new Action("Interface Type...") {
        		public void run() { 
        		new InterfaceTypeDialog(shell, undoContext, modelController).open();
        		}
			};
			newInterfaceAction.setImageDescriptor(getImageDescription(Constants.NEW_INTERFACE_TYPE_IMAGE_STRING));
		}
		return newInterfaceAction;
	}
	
	
	public Action getNewAttributeTypeAction() {
		if (newAttributeTypeAction == null) {
			newAttributeTypeAction =  new Action("Attribute Value Type...") {
				public void run() { 
					new AttributeValueTypeDialog(shell, undoContext, modelController).open();
				}
			};
			newAttributeTypeAction.setImageDescriptor(getImageDescription(Constants.NEW_ATTRIBUTE_VALUE_TYPE_IMAGE_STRING));
		}
		return newAttributeTypeAction;
	}
	
	public Action getNewFeatureTypeAction() {
		if (newFeatureTypeAction == null) {
			newFeatureTypeAction = new Action("Feature Type...") {
    			public void run() { 
    				new ComposableTypeDialog(shell, undoContext, modelController, true).open();
    			}
			};
			newFeatureTypeAction.setImageDescriptor(getImageDescription(Constants.NEW_FEATURE_TYPE_IMAGE_STRING));
		}
		return newFeatureTypeAction;
	}
	
	public Action getNewComponentTypeAction() {
		if (newComponentTypeAction == null) {
			newComponentTypeAction = new Action("Component Type...") {
    			public void run() { 
    				new ComposableTypeDialog(shell, undoContext, modelController, false).open();
    			}
			};	
			newComponentTypeAction.setImageDescriptor(getImageDescription(Constants.NEW_COMPONENT_TYPE_IMAGE_STRING));
		}
		return newComponentTypeAction;
	}
	
	
	public Action getSelectComponentRootAction() {
		if (selectComponentRootAction == null)
			selectComponentRootAction = new Action("Select Component Root...") {
	        	public void run() { 
	    			new SelectComponentRootDialog(shell, undoContext, modelController).open();
	    		}
	        };
	     return selectComponentRootAction;
	}
	
	public Action getSelectFeatureRootAction() {
		if (selectFeatureRootAction == null)
			selectFeatureRootAction = new Action("Select Feature Root...") {
	        	public void run() { 
	    			new SelectFeatureRootDialog(shell, undoContext, modelController).open();
	    		}
	        };
	     return selectFeatureRootAction;
	}
	
	public Action getPropertiesAction(final BasicController controller) {
		return new Action("Properties...") {
        	public void run() {
        		Dialog dialog = PropertyDialogFactory.getInstance().getPropertyDialog(shell, controller, modelController, undoContext);
        		if (dialog != null)
        			dialog.open();
        	}
        	
        }; 
	}
	
	public Action getDeleteAction(final BasicController controller) {
		return new Action("Delete") {
        	public void run() {
        		controller.delete();
        	}
        	
        }; 
	  
	}
	
	public Action getAddInterfaceDefinitionAction(final ComposableTypeController controller) {	
	
		return new Action("Add Interface...") {
				public void run() { 
					new InterfaceDefinitionDialog(shell, undoContext, controller, modelController).open();
				}
			};
       
	}
	
	public Action getAddAttributeDefinitionAction(final ComposableTypeController controller) {
		return new Action("Add Attribute...") {
        	public void run() {
        		new AttributeDefinitionDialog(shell, undoContext, controller, modelController).open();
        	}
        };

	}
	
	public Action getAddPartDefinitionAction(final ComposableTypeController controller, String actionName) {
	    return new Action(actionName){
        	public void run() {
	        	new PartDefinitionDialog(shell, undoContext, modelController, controller).open();
	     
        	}
        	
        };
   
	}


	
	public Action getAddPossibleTypesAction() {
	    return new Action("Add Possible Types...") {
        	public void run() {
	        	
        	}
        };
      
	}

	public Action getSetAsFeatureRootAction(final String rootName) {
	     return new Action("Set as Feature Root"){
	        	public void run() {
	        		IUndoableOperation operation = new SetFeatureRootOperation(modelController, rootName);
	        		operation.addContext(undoContext);
	            	try {
	            		OperationHistoryFactory.getOperationHistory().execute(operation, null, null);

	        		} catch (ExecutionException e) {
	        			// TODO Auto-generated catch block
	        			e.printStackTrace();
	        		}
	        		
	        	}
	        	
	        };    
	}
	
	public Action getSetAsComponentRootAction(final String rootName) {
	     return new Action("Set as Component Root"){
	        	public void run() {
	        		IUndoableOperation operation = new SetComponentRootOperation(modelController, rootName);
	        		operation.addContext(undoContext);
	            	try {
	            		OperationHistoryFactory.getOperationHistory().execute(operation, null, null);

	        		} catch (ExecutionException e) {
	        			// TODO Auto-generated catch block
	        			e.printStackTrace();
	        		}
	        		
	        	}
	        	
	        };    
	}

	public Action getAddConnectionAction(final ComponentTypeController controller) {
		return new Action("Connect Interfaces..") {		
			public void run() {
				new ConnectionDialog(shell, undoContext,  controller, modelController).open();
			}
			
		};
	}

	public Action getAddConstraintAction(final ComposableTypeController controller) {
	
			return new Action("Add Constraint...") {
				public void run() {
					new ConstraintDialog(shell, undoContext, modelController, false, (ComposableTypeController)controller).open();
				}
			};
		

	}
	
	public Action getAddImplementationConstraintAction(final FeatureTypeController controller) {	
			return new Action("Add Implementation Constraint...") {
				public void run() {
					new ConstraintDialog(shell, undoContext, modelController, true, controller).open();
				}
			};
	}
	

	public void updateActionStatus() {
		if (modelController.getModelType().equals(KoalishModel.MODEL_KIND)) {
			newFeatureTypeAction.setEnabled(false);	
		} else newFeatureTypeAction.setEnabled(true);
		
		if (modelController.getModelType().equals(ForfamelModel.MODEL_KIND)) {
			newInterfaceAction.setEnabled(false);
			newComponentTypeAction.setEnabled(false);			
			
		} else {		
			newInterfaceAction.setEnabled(true);
			newComponentTypeAction.setEnabled(true);
			
		}
		
		
		
	}
	
	private ImageDescriptor getImageDescription(String imageString) {
		return KumbangLabelProvider.getInstance().getImageDescriptor(imageString);
	}

	public Action getCreateComponentRootAction() {
		if (createComponentRootAction == null)
			createComponentRootAction = new Action("Create Root Component...") {
			public void run() {
				new ComposableTypeDialog(shell, undoContext, modelController, false, true).open();
				
			}
			
		};
		return createComponentRootAction;
	}
	
	public Action getCreateFeatureRootAction() {
		if (createFeatureRootAction == null)
			createFeatureRootAction = new Action("Create Root Feature...") {
			public void run() {
				new ComposableTypeDialog(shell, undoContext, modelController, true, true).open();
				
			}
			
		};
		return createFeatureRootAction;
	}
}
