/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.util;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.model.constraint.Expression;
import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.core.model.core.KumbangModel;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.Connection;
import kumbang.core.model.type.ElementaryDefinition;
import kumbang.core.model.type.ElementaryType;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.controllers.ConstraintController;

public class KumbangModelPrinter {

	private static KumbangModelPrinter instance = new KumbangModelPrinter();
	private boolean isFeatured = false;
	private boolean isComponented = false;
	
	private KumbangModelPrinter() {}
	
	
	public String getAsString(KumbangModel model, String type) {
		StringBuffer sb = new StringBuffer();
		appendModel(sb, model, type);
		
		return sb.toString();
	}
	
	public void appendModel(StringBuffer sb, KumbangModel model, String type) {
		if (type.equals(ForfamelModel.MODEL_KIND) || model.getComponentCount() == 0) {
			if (model.getFeatureRoot() == null)
				return;
			
			sb.append("Forfamel model "+ model.getName());
			sb.append("\n\troot feature " + model.getFeatureRoot().getName());
			isFeatured = true;
			isComponented = false;
		} else if (type.equals(KoalishModel.MODEL_KIND) || model.getFeatureCount() == 0) {
			if (model.getComponentRoot() == null)
				return;
			
			sb.append("Koalish model " + model.getName());
			sb.append("\n\troot component " + model.getComponentRoot().getName());
			isComponented = true;
			isFeatured = false;
		} else {
			if (model.getFeatureRoot() == null || model.getComponentRoot() == null)
				return;
			
			sb.append("Kumbang model " + model.getName());
			sb.append("\n\troot component " + model.getComponentRoot().getName());
			sb.append("\n\troot feature " + model.getFeatureRoot().getName());
			isFeatured = true;
			isComponented = true;
		}
		appendInterfaces(sb, model);
		appendComponents(sb, model);
		appendFeatures(sb, model);
		appendAttributes(sb, model);
	}

	private void appendAttributes(StringBuffer sb, KumbangModel model) {
		Iterator i = model.getAttributeValueTypes();
		if (i.hasNext())
			sb.append("\n\n//---attributes-----\n");
		while (i.hasNext())
			appendAttribute(sb, (AttributeValueType)i.next());
	}

	private void appendAttribute(StringBuffer sb, AttributeValueType type) {
		sb.append("\n\nattribute type ");
		appendName(sb, type);
		sb.append(" = {" );
		Iterator i = type.getValues();
		while (i.hasNext()) {
			sb.append(" ");
			appendName(sb, (Name)i.next());
			sb.append(i.hasNext() ? "," : " }");
		
		}
			sb.append("\n");
	}

	

	private void appendFeatures(StringBuffer sb, KumbangModel model) {
		if (!isFeatured)
			return;
		Iterator i = model.getFeatureTypes();
		if (i.hasNext())
			sb.append("\n\n//---features-----\n");
		while (i.hasNext())
			appendFeature(sb, (FeatureType)i.next());
			
		
	}

	private void appendFeature(StringBuffer sb, FeatureType type) {
		sb.append("\n\n");
		if (type.isAbstract())
			sb.append("abstract ");
		sb.append("feature type ");
		appendName(sb, type);
		Iterator i = type.getSupertypes();
		if (i.hasNext()) {
			sb.append(" extends");
			FeatureType next = (FeatureType) i.next();
			boolean many = i.hasNext();
			
			if (many)
				sb.append("(");
			else 
				sb.append(" ");
			appendName(sb, next);
			sb.append(i.hasNext() ? ", " : "");
			
			
			while(i.hasNext()) { 
				sb.append(" ");
				appendName(sb, (FeatureType)i.next());
				sb.append(i.hasNext() ? ", " : "");
			}
			if (many)
				sb.append(")");
			
		}
		sb.append(" {");
		
		i = type.getDirectParts();
		if (i.hasNext()) {
			sb.append("\n\tsubfeatures");
			while (i.hasNext())
				appendPartDefinition(sb, (PartDefinition)i.next());
			
		}
		
		i = type.getDirectAttributes();
		
		if (i.hasNext()) {
			sb.append("\n\tattributes");
		
			while (i.hasNext())
				appendAttributeDefinition(sb, (AttributeDefinition)i.next());
		}

		i = type.getConstraints();
		
		List<Expression> constraints = new LinkedList<Expression>();
		List<Expression> implementation = new LinkedList<Expression>();
		
				
		while (i.hasNext()) {
			Expression ex = (Expression)i.next();
			if (ConstraintController.isImplementation(ex))
				implementation.add(ex);
			else constraints.add(ex);
		}
		
		if (constraints.size() > 0) {
			sb.append("\n\tconstraints");
			for (Expression ex : constraints)
				appendConstraint(sb, ex, false);
		}
			
		if (implementation.size() > 0 && isComponented) {
			sb.append("\n\timplementation");
			for (Expression ex : implementation)
				appendConstraint(sb, ex, true);
		}
		
		
		sb.append("\n}\n\n");
	}

	private void appendConstraint(StringBuffer sb, Expression expression, boolean implementation) {
		sb.append("\n\t\t" + expression.getModelString(implementation) + ";");
		
	}


	private void appendComponents(StringBuffer sb, KumbangModel model) {
		if (!isComponented)
			return;
		Iterator i = model.getComponentTypes();
		if (i.hasNext())
			sb.append("\n\n//---components-----");
		while (i.hasNext())
			appendComponent(sb, (ComponentType)i.next());
	}

	private void appendComponent(StringBuffer sb, ComponentType type) {
		sb.append("\n\ncomponent type ");
		appendName(sb, type);
		sb.append(" {");
		Iterator i = type.getParts();
		if (i.hasNext())
			sb.append("\n\tcontains");
		while (i.hasNext())
			appendPartDefinition(sb, (PartDefinition)i.next());
	
		i = type.getInterfaces();
		List<InterfaceDefinition> required = new LinkedList<InterfaceDefinition>();
		List<InterfaceDefinition> provided = new LinkedList<InterfaceDefinition>();
		
		while(i.hasNext()) {
			InterfaceDefinition definition = (InterfaceDefinition)i.next();
			if (definition.isProvides())
				provided.add(definition);
			else required.add(definition);
		}
		
		if (provided.size() > 0) {
			sb.append("\n\tprovides");
			for (int j = 0; j < provided.size(); j++)
				appendInterfaceDefinition(sb, provided.get(j));
		}
		
		if (required.size() > 0) {
			sb.append("\n\trequires");
			for (int j = 0; j < required.size(); j++)
				appendInterfaceDefinition(sb, required.get(j));
		}
		
		i = type.getDirectAttributes();
		
		if (i.hasNext())
			sb.append("\n\tattributes");
		
		while (i.hasNext())
			appendAttributeDefinition(sb, (AttributeDefinition)i.next());
		

		i = type.getConnections();
		if (i.hasNext()) {
			sb.append("\n\tconnects");
			
			while (i.hasNext())
				appendConnection(sb, (Connection) i.next());
		}
		
		i = type.getConstraints();
		
		if (i.hasNext()) {
			sb.append("\n\tconstraints");
			
			while (i.hasNext())
				appendConstraint(sb, (Expression) i.next(), false);
		}
		
		sb.append("\n}");
	}

	private void appendConnection(StringBuffer sb, Connection connection) {
		if (!isComponented)
			return;
		sb.append("\n\t\t" + connection.toString() + ";");
	}


	private void appendAttributeDefinition(StringBuffer sb, AttributeDefinition definition) {
		sb.append("\n\t\t");
		sb.append(definition.getType().getName());
		sb.append(" ");
		appendName(sb, definition);
		sb.append(";");
		
	}




	private void appendInterfaceDefinition(StringBuffer sb, InterfaceDefinition definition) {
		if (!isComponented)
			return;
		Iterator i = definition.getPossibleTypes();
		InterfaceType first = (InterfaceType) i.next();
		
		String additional;
		if (definition.isGrounded()) {
			additional = " { grounded";
			if (definition.isOptional())
				additional += ", optional }";
			else additional += " }";
		} else if (definition.isOptional())
			additional = " { optional }";
		else additional = "";
		
		if (!i.hasNext()) {
			sb.append("\n\t\t");
			appendName(sb, first);
			sb.append(" ");
			appendName(sb, definition);
			sb.append(additional);
		}
		else {
			sb.append("\n\t\t(");
			appendName(sb, first);
			while (i.hasNext()) {
				sb.append(", ");
				appendName(sb, (InterfaceType)i.next());
			}
			sb.append(") ");
			appendName(sb, definition);
			sb.append(additional);
		}
		sb.append(";");
		
	}

	private void appendPartDefinition(StringBuffer sb, PartDefinition definition) {
			Iterator i = definition.getPossibleTypes();
			ComposableType first = (ComposableType) i.next();
			if (!i.hasNext()) {
				sb.append("\n\t\t");
				sb.append(first.getName());
				sb.append(" ");
			}
			else {
				sb.append("\n\t\t(");
				appendName(sb, first);
				while (i.hasNext()) {
					sb.append(", ");
					appendName(sb, (ComposableType)i.next());
				}
				sb.append(") ");
			}
			
		
			appendName(sb, definition);
			
			if (definition.getLowerBound() != 1 || definition.getUpperBound() != 1) {
				String simString = null;
				if (definition.getSimilarity() == PartDefinition.simDifferent)
					simString = "different";
				else if (definition.getSimilarity() == PartDefinition.simSame)
					simString = "same";
				
				sb.append("[" + definition.getLowerBound() + "-" + definition.getUpperBound() + "]" + ((simString == null) ? "" : " {" +  simString + "}"));
			
				
			}
			sb.append(";");
	}

	private void appendInterfaces(StringBuffer sb, KumbangModel model) {
		if (!isComponented)
			return;
		Iterator i = model.getInterfaceTypes();
		if (i.hasNext())
			sb.append("\n\n//---interfaces-----\n");
		while (i.hasNext())
			appendInterface(sb, (InterfaceType)i.next());
		
	}
	
	private void appendInterface(StringBuffer sb, InterfaceType type) {
		sb.append("\n\ninterface type ");
		appendName(sb, type);
		sb.append(" {\n");
		
		Iterator it = type.getFunctions();
		while(it.hasNext())  
			sb.append("\t" + it.next() + ";\n");
		
		sb.append("}\n");		
	}

	
	private void appendName(StringBuffer sb, Name name) {
		sb.append(name.getName());
		if (name.hasDisplayName())
			sb.append(" " + modifyName(name.getDisplayName()));
	}
	
	private void appendName(StringBuffer sb, ElementaryType type) {
		sb.append(type.getName());
		if (type.hasDisplayName())
			sb.append(" " + modifyName(type.getDisplayName()));
	}
	
	private String modifyName(String displayName) {
		if (displayName.startsWith("\"") && displayName.endsWith("\""))
			return displayName;
		return "\"" + displayName + "\"";
	}


	private void appendName(StringBuffer sb, ElementaryDefinition definition) {
		sb.append(definition.getName());
		if (definition.hasDisplayName())
			sb.append(" " + modifyName(definition.getDisplayName()));
		
	}
	
	public static KumbangModelPrinter getInstance() {
		return instance;
	}
}

